import * as React from 'react';
import { Table, Button, Header, Container } from 'semantic-ui-react';
import { ListContainer } from '@overflow/commons/react/component/ListContainer';

import Probe from '@overflow/probe/api/model/Probe';
import Sensor from '@overflow/sensor/api/model/Sensor';
import Target from '@overflow/target/api/model/Target';
import Domain from '@overflow/domain/api/model/Domain';
import Infra from '@overflow/infra/api/model/Infra';
import PageParams from '@overflow/commons/api/model/PageParams';
import Page from '@overflow/commons/api/model/Page';
import { Pager } from '@overflow/commons/react/component/Pager';
import { SortTableHeaderRow } from '@overflow/commons/react/component/SortTableHeaderRow';


export interface StateProps {
  probe?: Probe;
  target?: Target;
  sensorList?: Page;
  infraId?: number;
}

export interface DispatchProps {
  onReadAllByDomain?(domain: Domain, pageParams: PageParams): void;
  onReadAllByInfra?(infraId: number, pageParams: PageParams): void;
  onReadAllByProbe?(probe: Probe): void;
  onSelectSensor?(id: number): void;
  onAddSensor?(infraId: string): void;
}

export type SensorListProps = StateProps & DispatchProps;

export interface SensorListState {
  selected: Sensor;
}


export class SensorList extends React.Component<SensorListProps, SensorListState> {

  private countPerPage: number = 10;
  private sortCol: string = 'id';
  private sortDirection: string = 'descending';
  private pageNo: number = 0;

  constructor(props: SensorListProps, context: SensorListState) {
    super(props, context);
    this.state = {
      selected: null,
    };
  }

  public componentWillMount(): void {
    this.getData();
  }

  public getData(): void {
    const pageParams: PageParams = {
      pageNo: String(this.pageNo),
      countPerPage: String(this.countPerPage),
      sortCol: this.sortCol,
      sortDirection: this.sortDirection,
    };
    if (this.props.infraId === undefined) {
      let domain: Domain = {
        id: 1,
      };
      this.props.onReadAllByDomain(domain, pageParams);
    } else {
      this.props.onReadAllByInfra(this.props.infraId, pageParams);
    }
  }

  public handleSelect(selectedSensor: Sensor): void {
    this.setState({
      selected: selectedSensor,
    });

    this.props.onSelectSensor(selectedSensor.id);
  }

  public checkCellStatus(status: any): boolean {
    if (status.name === 'STOPPED') {
      return true;
    }
    return false;
  }

  public showStartStopBtn(status: any): JSX.Element {
    if (status.name === 'STARTED') {
      return <Button content='Stop' icon='stop' labelPosition='left' color={'blue'} floated={'right'} onClick={this.handleStartStop} />;
    } else {
      return <Button content='Start' icon='play' labelPosition='left' color={'blue'} floated={'right'} onClick={this.handleStartStop} />;
    }
  }

  public handleStartStop(event: any, data: any): void {
    console.log(event);
  }
  public handleAddSensor(): void {
    this.props.onAddSensor(this.props.infraId ? String(this.props.infraId) : '');
  }

  public render(): JSX.Element {

    let colsmap: Map<string, string> = new Map();
    colsmap.set('No.', 'id');
    colsmap.set('Status', 'status.name');
    colsmap.set('Crawler', 'crawler.name');
    colsmap.set('Description', 'description');
    colsmap.set('Item count', 'itemCount');

    let sensorList: JSX.Element = (
      <Container fluid>
        <Table celled selectable striped sortable>
          <Table.Header>
            <SortTableHeaderRow columnMap={colsmap} onHeaderColumnClick={this.handleSort} />
            {/* <Table.Row>
              <Table.HeaderCell textAlign={'center'}>No.</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Status</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Crawler</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Description</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Item count</Table.HeaderCell>
            </Table.Row> */}
          </Table.Header>

          <Table.Body>
            {this.props.sensorList ? this.props.sensorList.content.map((sensor: Sensor, index: number) => (
              <Table.Row key={index} onClick={this.handleSelect.bind(this, sensor)}>
                <Table.Cell textAlign={'center'}>{index + 1}</Table.Cell>
                <Table.Cell negative={this.checkCellStatus(sensor.status)} textAlign={'center'}>
                  {sensor.status.name}
                </Table.Cell>
                <Table.Cell><Header size='small'>{sensor.crawler.name}</Header></Table.Cell>
                <Table.Cell>{sensor.description}</Table.Cell>
                <Table.Cell>to do</Table.Cell>
                {/* <Table.Cell collapsing>{this.showStartStopBtn(sensor.metaSensorStatus)}</Table.Cell> */}
              </Table.Row>
            )) : ''}
          </Table.Body>
          <Table.Footer>
            <Table.Row>
              <Table.HeaderCell colSpan='6' textAlign='center'>
                <Pager page={this.props.sensorList} onPageChange={this.handlePaging} />
              </Table.HeaderCell>
            </Table.Row>
          </Table.Footer>
        </Table>
        <Button content='Add' icon='add' labelPosition='left' floated='right' positive onClick={this.handleAddSensor.bind(this)} />

      </Container>
    );

    return (
      <ListContainer
        contents={sensorList}
        data={this.props.sensorList}
      />
    );
  }

  private handlePaging = (pageNo: number) => {
    this.pageNo = pageNo;
    this.getData();
  }

  private handleSort = (col: string, direction: string): void => {
    console.log(col + ' || direction:' + direction);
    this.sortCol = col;
    this.sortDirection = direction;
    this.getData();
  }

}



