import * as React from 'react';
import { Table, Button, Modal, Input, Header, Container, InputOnChangeData, ButtonProps } from 'semantic-ui-react';
import { TargetDetail } from './TargetDetail';
import Infra from '@overflow/infra/api/model/Infra';
import Probe from '@overflow/probe/api/model/Probe';
import Domain from '@overflow/domain/api/model/Domain';
import { ListContainer } from '@overflow/commons/react/component/ListContainer';
import DiscoveryContainer from '@overflow/discovery/react/Discovery';
import Host from '@overflow/discovery/api/model/Host';
import Page from '@overflow/commons/api/model/Page';
import { Pager } from '@overflow/commons/react/component/Pager';
import * as Utils from '@overflow/commons/util/Utils';
import PageParams from '@overflow/commons/api/model/PageParams';
import { SortTableHeaderRow } from '@overflow/commons/react/component/SortTableHeaderRow';


export interface StateProps {
  probeId?: string;
  infraList?: Page;
}


export interface DispatchProps {
  onReadAllByProbe?(probe: Probe, pageParams: PageParams): void;
  onReadAllByDomain?(domain: Domain, pageParams: PageParams): void;
  onTargetSelection?(id: string): void;
}

export type Props = StateProps & DispatchProps;

export interface State {
  selected: Infra;
  openAddTarget: boolean;
}

export class TargetList extends React.Component<Props, State> {

  private countPerPage: number = 10;
  private sortCol: string = 'id';
  private sortDirection: string = 'descending';
  private pageNo: number = 0;

  constructor(props: Props, context: State) {
    super(props, context);
    this.state = {
      selected: null,
      openAddTarget: false,
    };
  }

  public componentWillMount(): void {
    this.getTargetList();
  }

  public getTargetList(): void {
    const pageParams: PageParams = {
      pageNo: String(this.pageNo),
      countPerPage: String(this.countPerPage),
      sortCol: this.sortCol,
      sortDirection: this.sortDirection,
    };

    if (this.props.probeId === undefined) {
      // FIXME: get domain
      let domain: Domain = {
        id: 1,
      };
      this.props.onReadAllByDomain(domain, pageParams);
    } else {
      let probe: Probe = {
        id: Number(this.props.probeId),
      };
      this.props.onReadAllByProbe(probe, pageParams);
    }
  }

  public handleSelect(selectedTarget: any): void {

    this.setState({
      selected: selectedTarget,
    });

    this.props.onTargetSelection(selectedTarget.id);
  }


  public handleAddTarget(event: React.MouseEvent<HTMLButtonElement>, data: ButtonProps): void {
    this.setState({
      openAddTarget: true,
    });
  }

  public handleSearch(result: Infra[]): void {
    // this.setState({
    //   list: result,
    // });
  }

  public onClosePopup = (bool: boolean) => {
    this.setState({ openAddTarget: bool });
  }

  public onRefreshList = () => {
    console.log('onRefreshList');
    this.setState({ openAddTarget: false });
    this.getTargetList();
  }

  public handlePaging = (pageNo: number) => {
    this.pageNo = pageNo;
    this.getTargetList();
  }

  public handleFilter(filterStr: string): void {
    // if (filterStr === null) {
    //   this.setState({
    //     list: this.data,
    //   });
    //   return;
    // }
    // let founds = new Array();
    // for (let probe of this.data) {
    //   if (probe.metaProbeStatus.name.indexOf(filterStr) !== -1) {
    //     founds.push(probe);
    //   }
    // }
    // this.setState({
    //   list: founds,
    // });
  }

  public render(): JSX.Element {
    let colsmap: Map<string, string> = new Map();
    colsmap.set('No.', 'id');
    colsmap.set('Type', 'infraType.name');
    colsmap.set('Name', 'target.displayName');
    colsmap.set('Sensor Count', 'sensorCount');
    colsmap.set('Created at', 'createDate');

    if (this.props.infraList === undefined) {
      return null;
    }
    let targetList =
      <Container fluid>
        <Button content='Discovery' icon='search' labelPosition='left' floated='right' positive onClick={this.handleAddTarget.bind(this)} />
        <br /><br />
        <Table celled selectable striped sortable>
          <Table.Header>
            <SortTableHeaderRow columnMap={colsmap} onHeaderColumnClick={this.handleSort} />
            {/* <Table.Row>
              <Table.HeaderCell textAlign={'center'}>No</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Type</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Name</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'} collapsing>Sensor Count</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Created at</Table.HeaderCell>
            </Table.Row> */}
          </Table.Header>

          <Table.Body>
            {this.props.infraList.content.map((infra: Infra, index: number) => (
              <Table.Row key={index} onClick={this.handleSelect.bind(this, infra)}>
                <Table.Cell textAlign={'center'}>{index + 1}</Table.Cell>
                <Table.Cell textAlign={'center'}>{infra.infraType.name}</Table.Cell>
                <Table.Cell textAlign={'center'}>{infra.target.displayName}</Table.Cell>
                <Table.Cell>TODO</Table.Cell>
                <Table.Cell>{Utils.date2date(infra.target.createDate)}</Table.Cell>
              </Table.Row>
            ))}
          </Table.Body>
          <Table.Footer>
            <Table.Row>
              <Table.HeaderCell colSpan='5'>
                <Pager page={this.props.infraList} onPageChange={this.handlePaging} />
              </Table.HeaderCell>
            </Table.Row>
          </Table.Footer>
        </Table>

        <DiscoveryContainer probeId={1} open={this.state.openAddTarget}
          onClosePopup={this.onClosePopup.bind(this)} onRefreshList={this.onRefreshList} />

      </Container>;
    return (
      <ListContainer
        contents={targetList}
        data={this.props.infraList}
        onSearch={this.handleSearch.bind(this)}
        filter={null}
      />
    );
  }

  private handleSelectedTarget = (infra: Infra): void => {
    console.log('handleSelectedTarget ');
    this.setState({ selected: infra });

    this.props.onTargetSelection(String(infra.id));
  }

  private handleSort = (col: string, direction: string): void => {
    console.log(col + ' || direction:' + direction);
    this.sortCol = col;
    this.sortDirection = direction;
    this.getTargetList();
  }
}

